//
// Sample DirectShow Sink Filter that accepts data for use in application
//
// Copyright Geraint Davies January 2000
//

#include "streams.h"

class CAppSink;	// forward reference
class CSinkInput : public CBaseInputPin
{
public:
    CSinkInput(CAppSink* pFilter, CCritSec*pLock, HRESULT*phr);

    // check if the pin can support this specific proposed type and format
    HRESULT CheckMediaType(const CMediaType *pmt);
    
    // returns the preferred formats for a pin
    HRESULT GetMediaType(int iPosition, CMediaType *pmt) {
	return VFW_S_NO_MORE_ITEMS;
	//and let the other pin make all the proposals
    }

    // these two methods can just forward the calls to the filter
    STDMETHODIMP Receive(IMediaSample *pSample);
    STDMETHODIMP EndOfStream(void);

private:
    CAppSink* m_pFilter;
};

//Then you need to define a filter class, something like this:
class CAppSink : public CBaseFilter
{
public:
    CAppSink(LPUNKNOWN pUnkOuter);
    ~CAppSink();


    int GetPinCount() { 
	return 1;
    }

    CBasePin *GetPin(int n) {
	    CAutoLock lock(&m_Lock);
	    if (n != 0) {
		return NULL;
    	    }
    	    if (!m_pInput) {
		HRESULT hr = S_OK;
	    	m_pInput = new CSinkInput(this, &m_Lock, &hr);
    	    }
    	    return m_pInput;
    }

    // called from pin -- override to process data
    HRESULT Receive(IMediaSample* pSample);

    // called from pin -- override if you care about EOS
    HRESULT EndOfStream();

private:
    CCritSec m_Lock;
    CSinkInput* m_pInput;
};



// object to build graph and manage process

class CAppSinkBuilder
{
public:
    CAppSinkBuilder();
    ~CAppSinkBuilder();


    HRESULT BuildFromFile(LPCWSTR pszFile);
    HRESULT Run();

private:
    void MakeSink();

    CAppSink* m_pSink;
    IGraphBuilder* m_pGraph;
};



